@@ -21,7 +21,8 @@ from pyspark.sql.types import (
     StringType,
     StructField,
     StructType,
-    TimestampType, VariantType,
+    TimestampType,
+    VariantType,
 )
 
 
@@ -33,8 +34,14 @@ class ConfigError(ValueError):
 class AuthConfig:
     """Authentication configuration for REST requests."""
 
-    type: Literal["none", "bearer"] = "none"
+    type: Literal["none", "bearer", "oauth2"] = "none"
     token: str | None = None
+    token_url: str | None = None
+    client_id: str | None = None
+    client_secret: str | None = None
+    scope: Tuple[str, ...] = field(default_factory=tuple)
+    audience: str | None = None
+    extra_params: Mapping[str, Any] = field(default_factory=dict)
 
 
 @dataclass(frozen=True)
@@ -196,11 +203,9 @@ def parse_config(
     if source.get("type") != "rest":
         raise ConfigError("Only REST sources are supported for now")
 
-    # token decides.
-    if token:
-        auth = AuthConfig(type="bearer", token=token)
-    else:
-        auth = AuthConfig()
+    runtime_options: Dict[str, Any] = dict(options or {})
+
+    auth = _parse_auth_config(source.get("auth"), token, runtime_options)
 
     base_url = source.get("base_url")
     if not isinstance(base_url, str) or not base_url:
@@ -213,8 +218,6 @@ def parse_config(
 
     stream = _parse_stream(stream_raw)
 
-    runtime_options = dict(options or {})
-
     return RestSourceConfig(
         version=version,
         base_url=base_url.rstrip("/"),
@@ -236,7 +239,20 @@ def config_to_dict(config: RestSourceConfig) -> Dict[str, Any]:
     }
     if config.auth.type == "bearer":
         # Expose only the auth type, never the token.
-        source["auth"] = {"type": config.auth.type}
+        source["auth"] = {"type": "bearer"}
+    elif config.auth.type == "oauth2":
+        auth_block: Dict[str, Any] = {"type": "oauth2"}
+        if config.auth.token_url:
+            auth_block["token_url"] = config.auth.token_url
+        if config.auth.client_id:
+            auth_block["client_id"] = config.auth.client_id
+        if config.auth.scope:
+            auth_block["scope"] = list(config.auth.scope)
+        if config.auth.audience:
+            auth_block["audience"] = config.auth.audience
+        if config.auth.extra_params:
+            auth_block["extra_params"] = dict(config.auth.extra_params)
+        source["auth"] = auth_block
 
     stream = config.stream
     stream_dict: Dict[str, Any] = {
@@ -313,15 +329,117 @@ def dump_config(config: RestSourceConfig) -> str:
     return yaml.safe_dump(data, sort_keys=False)
 
 
-def _parse_auth(auth: dict) -> AuthConfig:  # Deprecated – retained for backward compatibility (unused)
-    token = auth.get("token")
-    auth_type = auth.get("type")
+def _parse_auth_config(
+    raw_auth: Any,
+    runtime_token: Optional[str],
+    runtime_options: Dict[str, Any],
+) -> AuthConfig:
+    token_value = (
+        runtime_token.strip()
+        if isinstance(runtime_token, str) and runtime_token.strip()
+        else None
+    )
+
+    if raw_auth is None:
+        if token_value:
+            return AuthConfig(type="bearer", token=token_value)
+        return AuthConfig()
+
+    if not isinstance(raw_auth, Mapping):
+        raise ConfigError("'source.auth' must be a mapping when provided")
+
+    auth_type = raw_auth.get("type") or ("bearer" if token_value else "none")
+    if auth_type not in {"none", "bearer", "oauth2"}:
+        raise ConfigError(f"Unsupported auth type: {auth_type}")
+
+    if auth_type == "none":
+        return AuthConfig()
+
     if auth_type == "bearer":
-        if not isinstance(token, str):
-            raise ConfigError("'token' must be a string")
-        if token == "":
-            raise ConfigError("'token' cannot be empty")
-    return AuthConfig(type=auth_type, token=token)
+        raw_token = raw_auth.get("token")
+        raw_token = raw_token.strip() if isinstance(raw_token, str) else None
+        token = token_value or raw_token
+        if not token:
+            raise ConfigError(
+                "Bearer auth requires a token supplied via config or runtime options"
+            )
+        return AuthConfig(type="bearer", token=token)
+
+    # OAuth2 client credentials
+    token_url = raw_auth.get("token_url")
+    if not isinstance(token_url, str) or not token_url.strip():
+        raise ConfigError("'source.auth.token_url' must be provided for oauth2 auth")
+
+    client_id = raw_auth.get("client_id")
+    if not isinstance(client_id, str) or not client_id.strip():
+        raise ConfigError("'source.auth.client_id' must be provided for oauth2 auth")
+
+    client_secret = raw_auth.get("client_secret")
+    if isinstance(client_secret, str):
+        client_secret = client_secret.strip() or None
+        if (
+            client_secret
+            and client_secret.startswith("{{")
+            and client_secret.endswith("}}")
+        ):
+            client_secret = None
+    elif client_secret is not None:
+        client_secret = str(client_secret)
+
+    secret_from_options = runtime_options.pop("oauth_client_secret", None)
+    if isinstance(secret_from_options, str):
+        secret_from_options = secret_from_options.strip() or None
+    elif secret_from_options is not None:
+        secret_from_options = str(secret_from_options)
+
+    client_secret = client_secret or token_value or secret_from_options
+    if not client_secret:
+        raise ConfigError(
+            "OAuth2 auth requires a client secret supplied via config ('client_secret') or runtime option 'oauth_client_secret'",
+        )
+
+    scope_raw = raw_auth.get("scope")
+    scope: Tuple[str, ...] = ()
+    if isinstance(scope_raw, str):
+        scope = tuple(part for part in scope_raw.replace(",", " ").split() if part)
+    elif isinstance(scope_raw, (list, tuple)):
+        collected: List[str] = []
+        for item in scope_raw:
+            if not isinstance(item, str):
+                raise ConfigError("Each scope entry must be a string")
+            trimmed = item.strip()
+            if trimmed:
+                collected.append(trimmed)
+        scope = tuple(collected)
+    elif scope_raw not in (None, {}):
+        raise ConfigError("'source.auth.scope' must be a string or list of strings")
+
+    audience_raw = raw_auth.get("audience")
+    audience = (
+        audience_raw.strip()
+        if isinstance(audience_raw, str) and audience_raw.strip()
+        else None
+    )
+
+    extra_params_raw = raw_auth.get("extra_params")
+    extra_params: Dict[str, Any] = {}
+    if extra_params_raw is not None:
+        if not isinstance(extra_params_raw, Mapping):
+            raise ConfigError(
+                "'source.auth.extra_params' must be a mapping when provided"
+            )
+        for key, value in extra_params_raw.items():
+            extra_params[str(key)] = value
+
+    return AuthConfig(
+        type="oauth2",
+        token_url=token_url.strip(),
+        client_id=client_id.strip(),
+        client_secret=client_secret,
+        scope=scope,
+        audience=audience,
+        extra_params=extra_params,
+    )
 
 
 def _parse_stream(raw: Any) -> StreamConfig:
@@ -333,15 +451,17 @@ def _parse_stream(raw: Any) -> StreamConfig:
     # Check if we're using endpoint partitioning
     partition_data = raw.get("partition", {})
     using_endpoint_partitioning = (
-        isinstance(partition_data, dict) and
-        partition_data.get("strategy") == "endpoints" and
-        partition_data.get("endpoints")
+        isinstance(partition_data, dict)
+        and partition_data.get("strategy") == "endpoints"
+        and partition_data.get("endpoints")
     )
 
     # Only validate path if not using endpoint partitioning or if path is provided
     if path is None:
         if not using_endpoint_partitioning:
-            raise ConfigError("Stream 'path' is required unless using endpoint partitioning")
+            raise ConfigError(
+                "Stream 'path' is required unless using endpoint partitioning"
+            )
         # Use a placeholder path that will be overridden by endpoint partitioning
         path = "/"
     elif not isinstance(path, str) or not path.startswith("/"):
@@ -423,19 +543,31 @@ def _parse_pagination(raw: Any) -> PaginationConfig:
         raw.get("start_offset"), "pagination.start_offset", minimum=0, default=0
     )
     page_param = _maybe_str(raw.get("page_param"), "pagination.page_param")
-    start_page = _maybe_int(raw.get("start_page"), "pagination.start_page", minimum=1, default=1)
+    start_page = _maybe_int(
+        raw.get("start_page"), "pagination.start_page", minimum=1, default=1
+    )
     cursor_param = _maybe_str(raw.get("cursor_param"), "pagination.cursor_param")
     cursor_path = _maybe_path(raw.get("cursor_path"), "pagination.cursor_path")
     next_url_path = _maybe_path(raw.get("next_url_path"), "pagination.next_url_path")
     cursor_header = _maybe_str(raw.get("cursor_header"), "pagination.cursor_header")
     initial_cursor = _maybe_str(raw.get("initial_cursor"), "pagination.initial_cursor")
     stop_on_empty = _maybe_bool(
-        raw.get("stop_on_empty_response"), "pagination.stop_on_empty_response", default=True
+        raw.get("stop_on_empty_response"),
+        "pagination.stop_on_empty_response",
+        default=True,
+    )
+    total_pages_path = _maybe_path(
+        raw.get("total_pages_path"), "pagination.total_pages_path"
+    )
+    total_pages_header = _maybe_str(
+        raw.get("total_pages_header"), "pagination.total_pages_header"
+    )
+    total_records_path = _maybe_path(
+        raw.get("total_records_path"), "pagination.total_records_path"
+    )
+    total_records_header = _maybe_str(
+        raw.get("total_records_header"), "pagination.total_records_header"
     )
-    total_pages_path = _maybe_path(raw.get("total_pages_path"), "pagination.total_pages_path")
-    total_pages_header = _maybe_str(raw.get("total_pages_header"), "pagination.total_pages_header")
-    total_records_path = _maybe_path(raw.get("total_records_path"), "pagination.total_records_path")
-    total_records_header = _maybe_str(raw.get("total_records_header"), "pagination.total_records_header")
 
     if pag_type == "offset":
         if offset_param is None:
@@ -519,7 +651,11 @@ def _pagination_to_dict(config: PaginationConfig) -> Dict[str, Any]:
 
 
 def _maybe_int(
-    value: Any, field: str, *, minimum: Optional[int] = None, default: Optional[int] = None
+    value: Any,
+    field: str,
+    *,
+    minimum: Optional[int] = None,
+    default: Optional[int] = None,
 ) -> Optional[int]:
     if value is None:
         return default
@@ -604,15 +740,21 @@ def _parse_record_selector(raw: Any) -> RecordSelectorConfig:
         field_path = []
         for entry in field_path_raw:
             if not isinstance(entry, str) or not entry.strip():
-                raise ConfigError("Each entry in 'record_selector.field_path' must be a non-empty string")
+                raise ConfigError(
+                    "Each entry in 'record_selector.field_path' must be a non-empty string"
+                )
             field_path.append(entry.strip())
     else:
-        raise ConfigError("'record_selector.field_path' must be a list of strings or a string")
+        raise ConfigError(
+            "'record_selector.field_path' must be a list of strings or a string"
+        )
 
     record_filter = raw.get("record_filter")
     if record_filter is not None:
         if not isinstance(record_filter, str) or not record_filter.strip():
-            raise ConfigError("'record_selector.record_filter' must be a non-empty string when provided")
+            raise ConfigError(
+                "'record_selector.record_filter' must be a non-empty string when provided"
+            )
         record_filter = record_filter.strip()
 
     cast_to_schema_types = bool(raw.get("cast_to_schema_types", False))
@@ -639,16 +781,24 @@ def _parse_error_handler(raw: Any) -> ErrorHandlerConfig:
         retry_statuses = _default_retry_statuses()
     else:
         if not isinstance(retry_statuses_raw, list):
-            raise ConfigError("'error_handler.retry_statuses' must be a list when provided")
-        retry_statuses = tuple(_normalize_status_spec(value) for value in retry_statuses_raw)
+            raise ConfigError(
+                "'error_handler.retry_statuses' must be a list when provided"
+            )
+        retry_statuses = tuple(
+            _normalize_status_spec(value) for value in retry_statuses_raw
+        )
 
     retry_on_timeout = raw.get("retry_on_timeout", True)
     if not isinstance(retry_on_timeout, bool):
-        raise ConfigError("'error_handler.retry_on_timeout' must be a boolean when provided")
+        raise ConfigError(
+            "'error_handler.retry_on_timeout' must be a boolean when provided"
+        )
 
     retry_on_connection_errors = raw.get("retry_on_connection_errors", True)
     if not isinstance(retry_on_connection_errors, bool):
-        raise ConfigError("'error_handler.retry_on_connection_errors' must be a boolean when provided")
+        raise ConfigError(
+            "'error_handler.retry_on_connection_errors' must be a boolean when provided"
+        )
 
     backoff_raw = raw.get("backoff")
     if backoff_raw is None:
@@ -719,19 +869,27 @@ def _parse_partition(raw: Any) -> PartitionConfig:
     if strategy == "param_range":
         param = _maybe_str(raw.get("param"), "partition.param")
         if not param:
-            raise ConfigError("'partition.param' must be provided for param_range strategy")
+            raise ConfigError(
+                "'partition.param' must be provided for param_range strategy"
+            )
 
         range_start = raw.get("range_start")
         range_end = raw.get("range_end")
 
         if range_start is None:
-            raise ConfigError("'partition.range_start' must be provided for param_range strategy")
+            raise ConfigError(
+                "'partition.range_start' must be provided for param_range strategy"
+            )
         if range_end is None:
-            raise ConfigError("'partition.range_end' must be provided for param_range strategy")
+            raise ConfigError(
+                "'partition.range_end' must be provided for param_range strategy"
+            )
 
         range_kind = raw.get("range_kind", "numeric")
         if range_kind not in {"numeric", "date"}:
-            raise ConfigError("'partition.range_kind' must be either 'numeric' or 'date'")
+            raise ConfigError(
+                "'partition.range_kind' must be either 'numeric' or 'date'"
+            )
 
         # Parse step if provided
         if "range_step" in raw:
@@ -741,13 +899,19 @@ def _parse_partition(raw: Any) -> PartitionConfig:
             range_step = step_value
 
         # Optional templates
-        value_template = _maybe_str(raw.get("value_template"), "partition.value_template")
-        extra_template = _maybe_str(raw.get("extra_template"), "partition.extra_template")
+        value_template = _maybe_str(
+            raw.get("value_template"), "partition.value_template"
+        )
+        extra_template = _maybe_str(
+            raw.get("extra_template"), "partition.extra_template"
+        )
 
     elif strategy == "endpoints":
         raw_endpoints = raw.get("endpoints")
         if not raw_endpoints:
-            raise ConfigError("'partition.endpoints' must be provided for endpoints strategy")
+            raise ConfigError(
+                "'partition.endpoints' must be provided for endpoints strategy"
+            )
 
         if isinstance(raw_endpoints, str):
             # Handle comma-separated string format
@@ -760,11 +924,15 @@ def _parse_partition(raw: Any) -> PartitionConfig:
             endpoint_list = []
             for endpoint in raw_endpoints:
                 if not isinstance(endpoint, str) or not endpoint.strip():
-                    raise ConfigError("Each endpoint in 'partition.endpoints' must be a non-empty string")
+                    raise ConfigError(
+                        "Each endpoint in 'partition.endpoints' must be a non-empty string"
+                    )
                 endpoint_list.append(endpoint.strip())
             endpoints = tuple(endpoint_list)
         else:
-            raise ConfigError("'partition.endpoints' must be a list of strings or a comma-separated string")
+            raise ConfigError(
+                "'partition.endpoints' must be a list of strings or a comma-separated string"
+            )
 
     return PartitionConfig(
         strategy=strategy,
@@ -780,7 +948,6 @@ def _parse_partition(raw: Any) -> PartitionConfig:
     )
 
 
-
 def _normalize_status_spec(value: Any) -> str:
     if isinstance(value, int):
         code = value
@@ -797,7 +964,9 @@ def _normalize_status_spec(value: Any) -> str:
                 raise ConfigError("Pattern status codes must look like '5XX'")
             bucket = int(text[0])
             if bucket < 1 or bucket > 5:
-                raise ConfigError("Pattern status codes must be between '1XX' and '5XX'")
+                raise ConfigError(
+                    "Pattern status codes must be between '1XX' and '5XX'"
+                )
             return f"{bucket}XX"
         if text.isdigit():
             code = int(text)
@@ -845,7 +1014,10 @@ def parse_schema_struct(schema_text: str) -> StructType:
         try:
             return _parse_ddl_without_spark(schema_text)
         except Exception as fallback_exc:
-            raise ValueError(f"Unable to parse schema: {fallback_exc}") from original_exc
+            raise ValueError(
+                f"Unable to parse schema: {fallback_exc}"
+            ) from original_exc
+
 
 def _coerce_env(value: Any) -> Any:
     if isinstance(value, str) and value.startswith("${env:") and value.endswith("}"):
@@ -857,6 +1029,7 @@ def _coerce_env(value: Any) -> Any:
         return {key: _coerce_env(item) for key, item in value.items()}
     return value
 
+
 def _resolve_env(name: str) -> str:
     from os import getenv
 
