@@ -34,6 +34,7 @@ from pyspark.sql.types import (
 
 from .config import (
     AuthConfig,
+    ConfigError,
     ErrorHandlerConfig,
     PaginationConfig,
     RecordSelectorConfig,
@@ -115,12 +116,13 @@ class _RetryPolicy:
     def next_delay(self, retries_attempted: int) -> float:
         base = self._config.backoff.initial_delay_seconds
         multiplier = self._config.backoff.multiplier
-        delay = base * (multiplier ** retries_attempted)
+        delay = base * (multiplier**retries_attempted)
         max_delay = self._config.backoff.max_delay_seconds
         if max_delay > 0:
             delay = min(delay, max_delay)
         return max(delay, 0.0)
 
+
 @dataclass
 class RestClient:
     """HTTP client tailored for REST-to-DataFrame ingestion."""
@@ -134,8 +136,14 @@ class RestClient:
         headers = {"User-Agent": USER_AGENT}
         if self.auth.type == "bearer" and self.auth.token:
             headers["Authorization"] = f"Bearer {self.auth.token}"
+        elif self.auth.type == "oauth2":
+            access_token = self._obtain_oauth2_token()
+            headers["Authorization"] = f"Bearer {access_token}"
+            self._oauth2_token = access_token
 
-        self._client = httpx.Client(base_url=self.base_url, headers=headers, timeout=self.timeout)
+        self._client = httpx.Client(
+            base_url=self.base_url, headers=headers, timeout=self.timeout
+        )
 
     def close(self) -> None:
         self._client.close()
@@ -168,10 +176,14 @@ class RestClient:
             "raw_params": dict(stream.params or {}),
         }
 
-        rendered_params = {
-            key: _render_template(value, template_context)
-            for key, value in stream.params.items()
-        } if stream.params else {}
+        rendered_params = (
+            {
+                key: _render_template(value, template_context)
+                for key, value in stream.params.items()
+            }
+            if stream.params
+            else {}
+        )
 
         template_context["params"] = rendered_params
 
@@ -204,6 +216,56 @@ class RestClient:
             observe_records=observe_records,
         )
 
+    def _obtain_oauth2_token(self) -> str:
+        if not self.auth.token_url:
+            raise ConfigError("OAuth2 auth requires 'token_url'")
+        if not self.auth.client_id:
+            raise ConfigError("OAuth2 auth requires 'client_id'")
+
+        client_secret = self.auth.client_secret
+        if not client_secret:
+            secret = self.options.get("oauth_client_secret") if self.options else None
+            if isinstance(secret, str) and secret.strip():
+                client_secret = secret.strip()
+        if not client_secret:
+            raise ConfigError(
+                "OAuth2 auth requires a client secret provided via config or runtime option 'oauth_client_secret'",
+            )
+
+        data: Dict[str, Any] = {
+            "grant_type": "client_credentials",
+            "client_id": self.auth.client_id,
+            "client_secret": client_secret,
+        }
+        if self.auth.scope:
+            data["scope"] = " ".join(self.auth.scope)
+        if self.auth.audience:
+            data["audience"] = self.auth.audience
+        if self.auth.extra_params:
+            for key, value in self.auth.extra_params.items():
+                data[str(key)] = value
+
+        try:
+            response = httpx.post(self.auth.token_url, data=data, timeout=self.timeout)
+        except httpx.HTTPError as exc:
+            raise ConfigError(f"OAuth2 token request failed: {exc}") from exc
+
+        if response.status_code >= 400:
+            detail = response.text.strip()
+            raise ConfigError(
+                f"OAuth2 token request failed with status {response.status_code}: {detail or 'no response body'}",
+            )
+
+        try:
+            payload = response.json()
+        except ValueError as exc:  # pragma: no cover - unexpected server response
+            raise ConfigError("OAuth2 token response was not valid JSON") from exc
+
+        access_token = payload.get("access_token")
+        if not isinstance(access_token, str) or not access_token.strip():
+            raise ConfigError("OAuth2 token response missing 'access_token'")
+        return access_token.strip()
+
     def peek_page(self, stream: StreamConfig) -> Optional[RestPage]:
         """Return the first page without mutating incremental state."""
 
@@ -244,15 +306,25 @@ class RestClient:
 
         retry_policy = _RetryPolicy(stream.error_handler)
 
-        window_page = getattr(pagination_window, "page", None) if pagination_window else None
-        window_offset = getattr(pagination_window, "offset", None) if pagination_window else None
-        max_pages = getattr(pagination_window, "max_pages", None) if pagination_window else None
+        window_page = (
+            getattr(pagination_window, "page", None) if pagination_window else None
+        )
+        window_offset = (
+            getattr(pagination_window, "offset", None) if pagination_window else None
+        )
+        max_pages = (
+            getattr(pagination_window, "max_pages", None) if pagination_window else None
+        )
         extra_params = (
             dict(pagination_window.extra_params)
             if pagination_window and pagination_window.extra_params
             else {}
         )
-        path_override = getattr(pagination_window, "path_override", None) if pagination_window else None
+        path_override = (
+            getattr(pagination_window, "path_override", None)
+            if pagination_window
+            else None
+        )
 
         base_path = path_override or initial_path
 
@@ -271,13 +343,27 @@ class RestClient:
         page_number = pagination.start_page if pagination.type == "page" else 1
         if window_page is not None and pagination.type == "page":
             page_number = window_page
-        cursor_to_apply: Optional[str] = pagination.initial_cursor if pagination.type == "cursor" else None
+        cursor_to_apply: Optional[str] = (
+            pagination.initial_cursor if pagination.type == "cursor" else None
+        )
 
-        if pagination.type == "offset" and pagination.limit_param and pagination.page_size is not None:
+        if (
+            pagination.type == "offset"
+            and pagination.limit_param
+            and pagination.page_size is not None
+        ):
             query_params.setdefault(pagination.limit_param, pagination.page_size)
-        if pagination.type == "page" and pagination.limit_param and pagination.page_size is not None:
+        if (
+            pagination.type == "page"
+            and pagination.limit_param
+            and pagination.page_size is not None
+        ):
             query_params.setdefault(pagination.limit_param, pagination.page_size)
-        if pagination.type == "cursor" and pagination.cursor_param and cursor_to_apply is not None:
+        if (
+            pagination.type == "cursor"
+            and pagination.cursor_param
+            and cursor_to_apply is not None
+        ):
             query_params[pagination.cursor_param] = cursor_to_apply
             cursor_to_apply = None
 
@@ -314,7 +400,9 @@ class RestClient:
                 except json.JSONDecodeError as exc:
                     raise ValueError("Expected API response to be valid JSON") from exc
 
-                records = _extract_records(payload, stream.record_selector, declared_schema)
+                records = _extract_records(
+                    payload, stream.record_selector, declared_schema
+                )
                 if not isinstance(records, list):
                     raise ValueError("Expected API response to be a list of records")
 
@@ -360,7 +448,9 @@ class RestClient:
                 elif pagination.type == "cursor":
                     next_link = None
                     if pagination.next_url_path:
-                        next_link = _first_value_from_path(payload, pagination.next_url_path)
+                        next_link = _first_value_from_path(
+                            payload, pagination.next_url_path
+                        )
                     if isinstance(next_link, str) and next_link:
                         if pagination.stop_on_empty_response and not records:
                             next_url = None
@@ -369,7 +459,9 @@ class RestClient:
                         else:
                             parsed = urlparse(next_link)
                             # Include query params only when following a path without its own query.
-                            include_params = not (parsed.scheme or parsed.netloc or parsed.query)
+                            include_params = not (
+                                parsed.scheme or parsed.netloc or parsed.query
+                            )
                             next_url = next_link
                             cursor_to_apply = None
                     else:
@@ -414,7 +506,9 @@ class RestClient:
                     headers=request_headers,
                 )
             except (httpx.TimeoutException, httpx.RequestError) as exc:
-                if policy.should_retry_exception(exc) and policy.can_retry(retries_attempted):
+                if policy.should_retry_exception(exc) and policy.can_retry(
+                    retries_attempted
+                ):
                     delay = policy.next_delay(retries_attempted)
                     retries_attempted += 1
                     if delay > 0:
@@ -424,7 +518,9 @@ class RestClient:
 
             status_code = response.status_code
             if status_code >= 400:
-                if policy.should_retry_status(status_code) and policy.can_retry(retries_attempted):
+                if policy.should_retry_status(status_code) and policy.can_retry(
+                    retries_attempted
+                ):
                     delay = policy.next_delay(retries_attempted)
                     retries_attempted += 1
                     response.close()
@@ -531,7 +627,7 @@ def _filter_records(records: List[Any], expression: str) -> List[Any]:
         return records
     if expr not in _FILTER_CACHE:
         stripped = expr
-        if expr.startswith("{{") and expr.endswith("}}"): 
+        if expr.startswith("{{") and expr.endswith("}}"):
             stripped = expr[2:-2].strip()
         try:
             _FILTER_CACHE[expr] = _FILTER_ENV.compile_expression(stripped)
@@ -637,7 +733,9 @@ def _cast_value(value: Any, datatype: Any) -> Any:
         return value
     if isinstance(datatype, MapType):
         if isinstance(value, Mapping):
-            return {key: _cast_value(val, datatype.valueType) for key, val in value.items()}
+            return {
+                key: _cast_value(val, datatype.valueType) for key, val in value.items()
+            }
         return value
     if isinstance(datatype, StructType):
         if isinstance(value, Mapping):
@@ -795,7 +893,11 @@ class _IncrementalTracker:
                     self._initial_value = str(fallback)
 
     def apply_to_params(self, params: Dict[str, Any]) -> None:
-        if not self._enabled or self._initial_value is None or self._cursor_param is None:
+        if (
+            not self._enabled
+            or self._initial_value is None
+            or self._cursor_param is None
+        ):
             return
         params.setdefault(self._cursor_param, self._initial_value)
 
@@ -930,7 +1032,9 @@ def _read_remote_json(path: str) -> Dict[str, Any]:
     except FileNotFoundError:
         return {}
     except Exception as exc:  # pragma: no cover - depends on backend
-        raise RuntimeError(f"Failed to read incremental state from {path}: {exc}") from exc
+        raise RuntimeError(
+            f"Failed to read incremental state from {path}: {exc}"
+        ) from exc
 
     try:
         payload = json.loads(raw)
@@ -958,13 +1062,17 @@ def _write_remote_text(path: str, data: str) -> None:
                 except FileExistsError:
                     pass
     except Exception as exc:  # pragma: no cover - depends on backend
-        raise RuntimeError(f"Failed to prepare incremental state directory for {path}: {exc}") from exc
+        raise RuntimeError(
+            f"Failed to prepare incremental state directory for {path}: {exc}"
+        ) from exc
 
     try:
         with fs.open(remote_path, mode="w") as handle:
             handle.write(data)
     except Exception as exc:  # pragma: no cover - depends on backend
-        raise RuntimeError(f"Failed to write incremental state to {path}: {exc}") from exc
+        raise RuntimeError(
+            f"Failed to write incremental state to {path}: {exc}"
+        ) from exc
 
 
 def _get_remote_filesystem(path: str):
